// -*-c++-*-

/*!
  \file field_painter.cpp
  \brief field painter class Source File.
*/

/*
 *Copyright:

 Copyright (C) Hidehisa AKIYAMA

 This code is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 This code is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this code; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

 *EndCopyright:
 */

/////////////////////////////////////////////////////////////////////

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qt.h>

#include "field_painter.h"

#include "draw_config.h"
#include "view_config.h"

#include <rcsc/common/server_param.h>

#include <iostream>

/*-------------------------------------------------------------------*/
/*!

*/
FieldPainter::FieldPainter( const ViewConfig & conf )
    : M_conf( conf )
{

}

/*-------------------------------------------------------------------*/
/*!

*/
void
FieldPainter::draw( QPainter & painter )
{
    drawBackGround( painter );
    drawLines( painter );
    drawPenaltyAreaLines( painter );
    drawGoalAreaLines( painter );
    drawGoals( painter );
    if ( M_conf.isShownFlags() )
    {
        drawFlags( painter );
    }
}

/*-------------------------------------------------------------------*/
/*!

*/
void
FieldPainter::drawBackGround( QPainter & painter ) const
{
    // fill the whole region
    painter.fillRect( painter.window(),
                      DrawConfig::instance().fieldBrush() );

    ////////////////////////////////////////////////////////////////
    if ( M_conf.grassType() == ViewConfig::GRASS_LINES )
    {
        drawLinesGrass( painter );
    }
    else if ( M_conf.grassType() == ViewConfig::GRASS_CHECKER )
    {
        drawCheckerGrass( painter );
    }
}

/*-------------------------------------------------------------------*/
/*!

*/
void
FieldPainter::drawLinesGrass( QPainter & painter ) const
{
    const DrawConfig & dconf = DrawConfig::instance();

    const int left_x = ( M_conf.fieldCenter().x
                         - M_conf.scale( rcsc::ServerParam::i().pitchHalfLength() ) );
    const int top_y = ( M_conf.fieldCenter().y
                        - M_conf.scale( rcsc::ServerParam::i().pitchHalfWidth() ) );
    const int right_x = ( M_conf.fieldCenter().x
                          + M_conf.scale( rcsc::ServerParam::i().pitchHalfLength() ) );
    const int bottom_y = ( M_conf.fieldCenter().y
                           + M_conf.scale( rcsc::ServerParam::i().pitchHalfWidth() ) );

    const int grid = M_conf.scale( 10.0 ); // grid length

    painter.setPen( dconf.transparentPen() );
    //painter.setBrush( dconf.fieldDarkBrush() );

    int i = M_conf.fieldCenter().x - grid/2;
    int cnt = 0;
    while ( i > left_x )
    {
        if ( ! ( cnt % 2 ) )
        {
            int left = i - grid;
            painter.fillRect( left, top_y - grid/2,
                              i - left, bottom_y - top_y + grid,
                              dconf.fieldDarkBrush() );
        }
        i -= grid;
        ++cnt;
    }

    i = M_conf.fieldCenter().x + grid/2;
    cnt = 0;
    while ( i < right_x )
    {
        if ( ! ( cnt % 2 ) )
        {
            int right = i + grid;
            painter.fillRect( i, top_y - grid/2,
                              right - i, bottom_y - top_y + grid,
                              dconf.fieldDarkBrush() );
        }
        i += grid;
        ++cnt;
    }
}

/*-------------------------------------------------------------------*/
/*!

*/
void
FieldPainter::drawCheckerGrass( QPainter & painter )const
{
    const DrawConfig & dconf = DrawConfig::instance();

    painter.setPen( dconf.transparentPen() );
    painter.setBrush( dconf.fieldDarkBrush() );

    const int left_x
        = M_conf.fieldCenter().x
        - M_conf.scale( rcsc::ServerParam::i().pitchHalfLength() );
    const int top_y
        = M_conf.fieldCenter().y
        - M_conf.scale( rcsc::ServerParam::i().pitchHalfWidth() );
    const int right_x
        = M_conf.fieldCenter().x
        + M_conf.scale( rcsc::ServerParam::i().pitchHalfLength() );
    const int bottom_y
        = M_conf.fieldCenter().y
        + M_conf.scale( rcsc::ServerParam::i().pitchHalfWidth() );

    const int grid = M_conf.scale( 10.0 ); // grid length

    int flag = 0;
    for ( int i = M_conf.fieldCenter().x; i > left_x; i -= grid )
    {
        int cnt = flag;
        int left = i - grid;
        for ( int j = M_conf.fieldCenter().y; j > top_y; j -= grid )
        {
            if ( cnt % 2 )
            {
                int top = j - grid;
                painter.fillRect( left, top, i - left, j - top,
                                  dconf.fieldDarkBrush() );
            }
            ++cnt;
        }
        ++flag;
    }

    flag = 1;
    for ( int i = M_conf.fieldCenter().x;  i > left_x; i -= grid )
    {
        int cnt = flag;
        int left = i - grid;
        for ( int j = M_conf.fieldCenter().y; j < bottom_y; j += grid )
        {
            if ( cnt % 2 )
            {
                int bottom = j + grid;
                painter.fillRect( left, j, i - left, bottom - j,
                                  dconf.fieldDarkBrush() );
            }
            ++cnt;
        }
        ++flag;
    }
    flag = 0;
    for ( int i = M_conf.fieldCenter().x;  i < right_x; i += grid )
    {
        int cnt = flag;
        int right = i + grid;
        for ( int j = M_conf.fieldCenter().y; j < bottom_y; j += grid )
        {
            if ( cnt % 2 )
            {
                int bottom = j + grid;
                painter.fillRect( i, j, right - i, bottom - j,
                                  dconf.fieldDarkBrush() );
            }
            ++cnt;
        }
        ++flag;
    }
    flag = 1;
    for ( int i = M_conf.fieldCenter().x;  i < right_x; i += grid )
    {
        int cnt = flag;
        int right = i + grid;
        for ( int j = M_conf.fieldCenter().y; j > top_y; j -= grid )
        {
            if ( cnt % 2 )
            {
                int top = j - grid;
                painter.fillRect( i, top, right - i, j - top,
                                  dconf.fieldDarkBrush() );
            }
            ++cnt;
        }
        ++flag;
    }
}

/*-------------------------------------------------------------------*/
/*!

*/
void
FieldPainter::drawLines( QPainter & painter ) const
{
    const DrawConfig & dconf = DrawConfig::instance();

    // set gdi objects
    painter.setPen( dconf.linePen() );
    painter.setBrush( dconf.transparentBrush() );

    // set screen coordinates of field
    int left_x   = M_conf.screenX( - rcsc::ServerParam::i().pitchHalfLength() );
    int right_x  = M_conf.screenX( + rcsc::ServerParam::i().pitchHalfLength() );
    int top_y    = M_conf.screenY( - rcsc::ServerParam::i().pitchHalfWidth() );
    int bottom_y = M_conf.screenY( + rcsc::ServerParam::i().pitchHalfWidth() );

    // side lines & goal lines
    painter.drawLine( left_x, top_y, right_x, top_y );
    painter.drawLine( right_x, top_y, right_x, bottom_y );
    painter.drawLine( right_x, bottom_y, left_x, bottom_y );
    painter.drawLine( left_x, bottom_y, left_x, top_y );

    if ( M_conf.keepawayMode()
         || rcsc::ServerParam::i().keepawayMode() )
    {
        // keepaway area
        int ka_left = M_conf.screenX( - rcsc::ServerParam::i().keepawayLength() * 0.5 );
        int ka_top = M_conf.screenY( - rcsc::ServerParam::i().keepawayWidth() * 0.5 );
        int ka_width = M_conf.scale( rcsc::ServerParam::i().keepawayWidth() );
        int ka_length = M_conf.scale( rcsc::ServerParam::i().keepawayWidth() );

        painter.drawRect( ka_left, ka_top, ka_length, ka_width );
    }
    else
    {
        // center line
        painter.drawLine( M_conf.fieldCenter().x, top_y,
                          M_conf.fieldCenter().x, bottom_y );

        // center circle
        int center_radius = M_conf.scale( rcsc::ServerParam::i().centerCircleR() );
        painter.drawEllipse( M_conf.fieldCenter().x - center_radius,
                             M_conf.fieldCenter().y - center_radius,
                             center_radius * 2,
                             center_radius * 2 );
    }
}

/*-------------------------------------------------------------------*/
/*!

*/
void
FieldPainter::drawPenaltyAreaLines( QPainter & painter ) const
{
    const DrawConfig & dconf = DrawConfig::instance();
    // set gdi objects
    painter.setPen( dconf.linePen() );
    painter.setBrush( dconf.transparentBrush() );

    // set screen coordinates of field
    int left_x   = M_conf.screenX( - rcsc::ServerParam::i().pitchHalfLength() );
    int right_x  = M_conf.screenX( + rcsc::ServerParam::i().pitchHalfLength() );

    // set penalty area params
    int pen_top_y    = M_conf.screenY( - rcsc::ServerParam::i().penaltyAreaHalfWidth() );
    int pen_bottom_y = M_conf.screenY( + rcsc::ServerParam::i().penaltyAreaHalfWidth() );
    double pen_circle_y_degree_abs
        = std::acos( ( rcsc::ServerParam::i().penaltyAreaLength()
                       - rcsc::ServerParam::DEFAULT_PENALTY_SPOT_DIST )
                     / rcsc::ServerParam::DEFAULT_PENALTY_CIRCLE_R )
        * ( 180.0 / M_PI );
    int span_angle = qRound( pen_circle_y_degree_abs * 2.0 * 16 );
    int pen_circle_r = M_conf.scale( rcsc::ServerParam::DEFAULT_PENALTY_CIRCLE_R );
    int pen_circle_size = M_conf.scale( rcsc::ServerParam::DEFAULT_PENALTY_CIRCLE_R * 2.0 );

    // left penalty area X
    int pen_x = M_conf.screenX( -( rcsc::ServerParam::i().pitchHalfLength()
                                   - rcsc::ServerParam::i().penaltyAreaLength() ) );
    // left arc
    int pen_spot_x = M_conf.screenX( -( rcsc::ServerParam::i().pitchHalfLength()
                                        - rcsc::ServerParam::DEFAULT_PENALTY_SPOT_DIST ) );
    painter.drawArc( pen_spot_x - pen_circle_r + 1,
                     M_conf.fieldCenter().y - pen_circle_r,
                     pen_circle_size,
                     pen_circle_size,
                     qRound( -pen_circle_y_degree_abs * 16 ),
                     span_angle );
    // left rectangle
    painter.drawLine( left_x, pen_top_y, pen_x, pen_top_y );
    painter.drawLine( pen_x, pen_top_y, pen_x, pen_bottom_y );
    painter.drawLine( pen_x, pen_bottom_y, left_x, pen_bottom_y );
    // left spot
    painter.drawPoint( pen_spot_x, M_conf.fieldCenter().y );

    // right penalty area X
    pen_x = M_conf.screenX( +( rcsc::ServerParam::i().pitchHalfLength()
                               - rcsc::ServerParam::i().penaltyAreaLength() ) );
    // right arc
    pen_spot_x = M_conf.screenX( +( rcsc::ServerParam::i().pitchHalfLength()
                                    - rcsc::ServerParam::DEFAULT_PENALTY_SPOT_DIST ) );
    painter.drawArc( pen_spot_x - pen_circle_r,
                     M_conf.fieldCenter().y - pen_circle_r,
                     pen_circle_size, pen_circle_size,
                     qRound( ( 180.0 - pen_circle_y_degree_abs + 0.5 ) * 16 ),
                     span_angle );
    // right rectangle
    painter.drawLine( right_x, pen_top_y, pen_x, pen_top_y );
    painter.drawLine( pen_x, pen_top_y, pen_x, pen_bottom_y );
    painter.drawLine( pen_x, pen_bottom_y, right_x, pen_bottom_y );
    // right spot
    painter.drawPoint( pen_spot_x, M_conf.fieldCenter().y );
}

/*-------------------------------------------------------------------*/
/*!

*/
void
FieldPainter::drawGoalAreaLines( QPainter & painter ) const
{
    const DrawConfig & dconf = DrawConfig::instance();
    // set gdi objects
    painter.setPen( dconf.linePen() );
    painter.setBrush( dconf.transparentBrush() );

    // set screen coordinates of field
    int left_x   = M_conf.screenX( - rcsc::ServerParam::i().pitchHalfLength() );
    int right_x  = M_conf.screenX( + rcsc::ServerParam::i().pitchHalfLength() );

    // set coordinates M_confs
    int goal_area_y_abs = M_conf.scale( rcsc::ServerParam::i().goalAreaHalfWidth() );
    int goal_area_top_y = M_conf.fieldCenter().y - goal_area_y_abs;
    int goal_area_bottom_y = M_conf.fieldCenter().y + goal_area_y_abs;

    // left goal area
    int goal_area_x = M_conf.screenX( - rcsc::ServerParam::i().pitchHalfLength()
                                      + rcsc::ServerParam::i().goalAreaLength() );
    painter.drawLine( left_x, goal_area_top_y, goal_area_x, goal_area_top_y );
    painter.drawLine( goal_area_x, goal_area_top_y, goal_area_x, goal_area_bottom_y );
    painter.drawLine( goal_area_x, goal_area_bottom_y, left_x, goal_area_bottom_y );

    // right goal area
    goal_area_x = M_conf.screenX( rcsc::ServerParam::i().pitchHalfLength()
                                  - rcsc::ServerParam::i().goalAreaLength() );
    painter.drawLine( right_x, goal_area_top_y, goal_area_x, goal_area_top_y );
    painter.drawLine( goal_area_x, goal_area_top_y, goal_area_x, goal_area_bottom_y );
    painter.drawLine( goal_area_x, goal_area_bottom_y, right_x, goal_area_bottom_y );
}

/*-------------------------------------------------------------------*/
/*!

*/
void
FieldPainter::drawGoals( QPainter & painter ) const
{
    // set gdi objects
    painter.setPen( Qt::black );
    painter.setBrush( Qt::black );

    // set coordinates param
    int goal_top_y = M_conf.screenY( - rcsc::ServerParam::i().goalHalfWidth() );
    int goal_size_x = M_conf.scale( rcsc::ServerParam::i().goalDepth() );
    int goal_size_y = M_conf.scale( rcsc::ServerParam::i().goalWidth() );

    int post_top_y = M_conf.screenY( - rcsc::ServerParam::i().goalHalfWidth()
                                     - rcsc::ServerParam::i().goalPostRadius() * 2.0 );
    int post_bottom_y = M_conf.screenY( + rcsc::ServerParam::i().goalHalfWidth() );
    int post_diameter = M_conf.scale( rcsc::ServerParam::i().goalPostRadius() * 2.0 );

    // left goal
    painter.drawRect( M_conf.screenX( - rcsc::ServerParam::i().pitchHalfLength()
                                      - rcsc::ServerParam::i().goalDepth() ) - 1,
                      goal_top_y,
                      goal_size_x,
                      goal_size_y );
    if ( post_diameter >= 1 )
    {
        int post_x = M_conf.screenX( - rcsc::ServerParam::i().pitchHalfLength() );
        painter.drawEllipse( post_x,
                             post_top_y,
                             post_diameter,
                             post_diameter );
        painter.drawEllipse( post_x,
                             post_bottom_y,
                             post_diameter,
                             post_diameter );
    }
    // right goal
    painter.drawRect( M_conf.screenX( rcsc::ServerParam::i().pitchHalfLength() ) + 1,
                      goal_top_y,
                      goal_size_x,
                      goal_size_y );
    if ( post_diameter >= 1 )
    {
        int post_x = M_conf.screenX( rcsc::ServerParam::i().pitchHalfLength()
                                     - rcsc::ServerParam::i().goalPostRadius() * 2.0 );
        painter.drawEllipse( post_x,
                             post_top_y,
                             post_diameter,
                             post_diameter );
        painter.drawEllipse( post_x,
                             post_bottom_y,
                             post_diameter,
                             post_diameter );
    }
}

/*-------------------------------------------------------------------*/
/*!

*/
void
FieldPainter::drawFlags( QPainter & painter ) const
{
    // set gdi objects
    painter.setPen( DrawConfig::instance().linePen() );
    painter.setBrush( DrawConfig::instance().transparentBrush() );

    // set size or coordinates params
    int flag_radius = M_conf.scale( 0.5 );
    if ( flag_radius < 2 ) flag_radius = 2;
    if ( flag_radius > 5 ) flag_radius = 5;
    int flag_diameter = flag_radius * 2;

    int x, y;
    int pitch_half_length = M_conf.scale( rcsc::ServerParam::i().pitchHalfLength() );
    int pitch_half_width = M_conf.scale( rcsc::ServerParam::i().pitchHalfWidth() );
    int pitch_margin_x = M_conf.scale( rcsc::ServerParam::i().pitchHalfLength()
                                       + rcsc::ServerParam::DEFAULT_PITCH_MARGIN );
    int pitch_margin_y = M_conf.scale( rcsc::ServerParam::i().pitchHalfWidth()
                                       + rcsc::ServerParam::DEFAULT_PITCH_MARGIN );
    int penalty_x = M_conf.scale( rcsc::ServerParam::i().pitchHalfLength()
                                  - rcsc::ServerParam::i().penaltyAreaLength() );
    int penalty_y = M_conf.scale( rcsc::ServerParam::i().penaltyAreaHalfWidth() );
    int goal_y = M_conf.scale( rcsc::ServerParam::i().goalHalfWidth() );
    int scale10 = M_conf.scale( 10.0 );
    int scale20 = M_conf.scale( 20.0 );
    int scale30 = M_conf.scale( 30.0 );
    int scale40 = M_conf.scale( 40.0 );
    int scale50 = M_conf.scale( 50.0 );

    // goal left
    x = M_conf.fieldCenter().x - pitch_half_length - flag_radius;
    y = M_conf.fieldCenter().y - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // goal right
    x = M_conf.fieldCenter().x + pitch_half_length - flag_radius;
    y = M_conf.fieldCenter().y - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag c
    x  = M_conf.fieldCenter().x - flag_radius;
    y = M_conf.fieldCenter().y - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag c t
    x = M_conf.fieldCenter().x - flag_radius;
    y = M_conf.fieldCenter().y - pitch_half_width - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );

    // flag c b
    x = M_conf.fieldCenter().x - flag_radius;
    y = M_conf.fieldCenter().y + pitch_half_width - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag l t
    x = M_conf.fieldCenter().x - pitch_half_length - flag_radius;
    y = M_conf.fieldCenter().y - pitch_half_width - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag l b
    x = M_conf.fieldCenter().x - pitch_half_length - flag_radius;
    y = M_conf.fieldCenter().y + pitch_half_width - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag r t
    x = M_conf.fieldCenter().x + pitch_half_length - flag_radius;
    y = M_conf.fieldCenter().y - pitch_half_width - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag r b
    x = M_conf.fieldCenter().x + pitch_half_length - flag_radius;
    y = M_conf.fieldCenter().y + pitch_half_width - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag p l t
    x = M_conf.fieldCenter().x - penalty_x - flag_radius;
    y = M_conf.fieldCenter().y - penalty_y - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag p l c
    x = M_conf.fieldCenter().x - penalty_x - flag_radius;
    y = M_conf.fieldCenter().y - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag p l b
    x = M_conf.fieldCenter().x - penalty_x - flag_radius;
    y = M_conf.fieldCenter().y + penalty_y - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag p r t
    x = M_conf.fieldCenter().x + penalty_x - flag_radius;
    y = M_conf.fieldCenter().y - penalty_y - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag p r c
    x = M_conf.fieldCenter().x + penalty_x - flag_radius;
    y = M_conf.fieldCenter().y - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag p r b
    x = M_conf.fieldCenter().x + penalty_x - flag_radius;
    y = M_conf.fieldCenter().y + penalty_y - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );

    // flag g l t
    x = M_conf.fieldCenter().x - pitch_half_length - flag_radius;
    y = M_conf.fieldCenter().y - goal_y - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag g l b
    x = M_conf.fieldCenter().x - pitch_half_length - flag_radius;
    y = M_conf.fieldCenter().y + goal_y - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag g r t
    x = M_conf.fieldCenter().x + pitch_half_length - flag_radius;
    y = M_conf.fieldCenter().y - goal_y - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag g r b
    x = M_conf.fieldCenter().x + pitch_half_length - flag_radius;
    y = M_conf.fieldCenter().y + goal_y - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );

    // flag t ...

    y = M_conf.fieldCenter().y - pitch_margin_y - flag_radius;
    // flag t l 50
    x = M_conf.fieldCenter().x - scale50 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag t l 40
    x = M_conf.fieldCenter().x - scale40 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag t l 30
    x = M_conf.fieldCenter().x - scale30 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag t l 20
    x = M_conf.fieldCenter().x - scale20 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag t l 10
    x = M_conf.fieldCenter().x - scale10 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag t 0
    x = M_conf.fieldCenter().x - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag t r 10
    x = M_conf.fieldCenter().x + scale10 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag t r 20
    x = M_conf.fieldCenter().x + scale20 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag t r 30
    x = M_conf.fieldCenter().x + scale30 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag t r 40
    x = M_conf.fieldCenter().x + scale40 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag t r 50
    x = M_conf.fieldCenter().x + scale50 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );

    // flag b ...

    y = M_conf.fieldCenter().y + pitch_margin_y - flag_radius;
    // flag b l 50
    x = M_conf.fieldCenter().x - scale50 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag b l 40
    x = M_conf.fieldCenter().x - scale40 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag b l 30
    x = M_conf.fieldCenter().x - scale30 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag b l 20
    x = M_conf.fieldCenter().x - scale20 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag b l 10
    x = M_conf.fieldCenter().x - scale10 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag b 0
    x = M_conf.fieldCenter().x - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag b r 10
    x = M_conf.fieldCenter().x + scale10 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag b r 20
    x = M_conf.fieldCenter().x + scale20 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag b r 30
    x = M_conf.fieldCenter().x + scale30 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag b r 40
    x = M_conf.fieldCenter().x + scale40 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag b r 50
    x = M_conf.fieldCenter().x + scale50 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );

    // flag l ...

    x = M_conf.fieldCenter().x - pitch_margin_x - flag_radius;
    // flag l t 30
    y = M_conf.fieldCenter().y - scale30 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag l t 20
    y = M_conf.fieldCenter().y - scale20 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag l t 10
    y = M_conf.fieldCenter().y - scale10 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag l 0
    y = M_conf.fieldCenter().y - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag l b 10
    y = M_conf.fieldCenter().y + scale10 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag l b 20
    y = M_conf.fieldCenter().y + scale20 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag l b 30
    y = M_conf.fieldCenter().y + scale30 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );

    // flag r ...

    x = M_conf.fieldCenter().x + pitch_margin_x - flag_radius;
    // flag r t 30
    y = M_conf.fieldCenter().y - scale30 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag r t 20
    y = M_conf.fieldCenter().y - scale20 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag r t 10
    y = M_conf.fieldCenter().y - scale10 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag r 0
    y = M_conf.fieldCenter().y - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag r b 10
    y = M_conf.fieldCenter().y + scale10 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag r b 20
    y = M_conf.fieldCenter().y + scale20 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
    // flag r b 30
    y = M_conf.fieldCenter().y + scale30 - flag_radius;
    painter.drawEllipse( x, y, flag_diameter, flag_diameter );
}
